@@ -10,14 +10,17 @@ implicit or backwardeuler
 trapezoidal or cranknicolson
 """
 import math
-import numpy as np
 import sys
 import time
 
+import numpy as np
+from scipy.optimize import newton
+
 # from scipy.sparse import csc_matrix
 # import scipy.sparse.linalg as splinalg
 # from numpy.linalg import inv
 import flowdyn.field as field
+from flowdyn.monitors import monitor
 
 # --------------------------------------------------------------------
 # portage
@@ -26,11 +29,10 @@ if float(sys.version[:3]) >= 3.3:  # or 3.8
     myclock = time.process_time
 else:
     myclock = time.clock
-start = myclock()
 
 # --------------------------------------------------------------------
 
-# --------------------------------------------------------------------
+
 class fakemodel:
     """ """
 
@@ -68,14 +70,16 @@ class fakedisc:
 # generic model
 # --------------------------------------------------------------------
 
-
 class timemodel:
     """ """
 
-    def __init__(self, mesh, modeldisc):
+    def __init__(self, mesh, modeldisc, monitors={}):
         self.mesh = mesh
         self.modeldisc = modeldisc
+        self.monitors = monitors
         self.reset()
+        # define function for monitoring
+        self._monitordict = { 'residual': self.mon_residual }
 
     def reset(self):
         """ """
@@ -92,16 +96,17 @@ class timemodel:
         """
         self.residual = self.modeldisc.rhs(field)
 
-    def step(self, f, dt):
+    def step(self, f, dtloc):
         """virtual method for one step integration
 
         Args:
-          f:
-          dt:
+          f: field to compute 
+          dt: time array or scalar
 
         Returns:
+            returns dQ/dt 
         """
-        raise NameError("not implemented for virtual class")
+        pass #raise NameError("not implemented for virtual class")
 
     def add_res(self, f, dt, subtimecoef=1.0):
         """
@@ -118,24 +123,46 @@ class timemodel:
         for i in range(f.neq):
             f.data[i] += dt * self.residual[i]  # time can be scalar or np.array
 
-    def solve(self, f, condition, tsave, flush=None):
+    def _check_end(self, stop):
         """
+        """
+        check_end = {}
+        for key, value in stop.items():
+            if key=='tottime':
+                check_end[key] = self._time >= value
+            if key=='maxit':
+                check_end[key] = self._nit >= value
+        return any(check_end.values())
+
+    def _parse_monitors(self, monitors):
+        for monkey, monval in monitors.items():
+            if monkey in self._monitordict.keys():
+                self._monitordict[monkey](monval)
+            else:
+                raise NameError("unknown monitor key: "+monkey)
+
+
+    def solve_legacy(self, f, condition, tsave, 
+            stop=None, flush=None, monitors={}):
+        """Solve dQ/dt=RHS(Q,t)
 
         Args:
-          f:
-          condition:
-          tsave:
+          f: initial field
+          condition: CFL number
+          tsave: array/list of time to save
           flush:  (Default value = None)
 
         Returns:
-
+          list of solution fields (size of tsave)
         """
-        self._nit = 0
+        self.reset() # reset cputime and nit
         self.condition = condition
+        # initialization before loop
         itfield = f.copy()
         if flush:
             alldata = [d for d in itfield.data]
         results = []
+        start = myclock()
         for t in np.arange(len(tsave)):
             endcycle = 0
             while endcycle == 0:
@@ -156,6 +183,66 @@ class timemodel:
             np.save(flush, alldata)
         return results
 
+    def solve(self, f, condition, tsave=[], 
+            stop=None, flush=None, monitors={}):
+        """Solve dQ/dt=RHS(Q,t)
+
+        Args:
+          f: initial field
+          condition: CFL number
+          tsave: array/list of time to save
+          flush:  (Default value = None)
+
+        Returns:
+          list of solution fields (size of tsave)
+        """
+        self.reset() # reset cputime and nit
+        self.condition = condition
+        stopcrit = { 'tottime': tsave[-1] } if len(tsave)>0 else {}
+        if stop is not None: stopcrit.update(stop)
+        if not stopcrit:
+            raise ValueError("missing stopping criteria")
+        monitors = { **self.monitors, **monitors }
+        # initialization before loop
+        self.Qn = f.copy()
+        self._time = self.Qn.time
+        if flush:
+            alldata = [d for d in self.Qn.data]
+        results = []
+        start = myclock()
+        isave, nsave = 0, len(tsave)
+        # loop testing all ending criteria
+        checkend = self._check_end(stopcrit)
+        self._parse_monitors(monitors)
+        while not checkend:
+            dtloc = self.modeldisc.calc_timestep(self.Qn, condition)
+            mindtloc = min(dtloc)
+            Qnn = self.Qn.copy()
+            if isave < nsave:
+                if self.Qn.time+mindtloc >= tsave[isave]:
+                    # compute smaller step with same integrator
+                    self.step(Qnn, tsave[isave]-self.Qn.time)
+                    results.append(Qnn)
+                    #results.append(self.Qn.interpol(Qnn, tsave[isave]))
+                    isave += 1
+                    # step back to self.Qn
+                    Qnn = self.Qn.copy()
+            self.step(Qnn, mindtloc)
+            self.Qn = Qnn
+            self._nit += 1
+            self._time = self.Qn.time
+            self._parse_monitors(monitors)
+            if flush:
+                for i, q in zip(range(len(alldata)), self.Qn.data):
+                    alldata[i] = np.vstack((alldata[i], q))
+            checkend = self._check_end(stopcrit)
+            if checkend and len(results)==0:
+                results.append(self.Qn)
+        self._cputime = myclock() - start
+        if flush:
+            np.save(flush, alldata)
+        return results
+
     def nit(self):
         """returns number of computed iterations"""
         return self._nit
@@ -164,16 +251,34 @@ class timemodel:
         """returns cputime"""
         return self._cputime
 
+    def perf_micros(self):
+        """returns perf in µs"""
+        return self._cputime * 1.0e6 / self._nit / self.modeldisc.nelem
+
     def show_perf(self):
         """print performance"""
         print(
             "cpu time computation ({0:d} it) : {1:.3f}s\n  {2:.2f} µs/cell/it".format(
                 self._nit,
                 self._cputime,
-                self._cputime * 1.0e6 / self._nit / self.modeldisc.nelem,
+                self.perf_micros(),
             )
         )
 
+    def mon_residual(self, params: dict):
+        """compute residual average and monitor it
+
+        Args:
+            params (dict): [description]
+        """
+        if self._nit % params.get('frequency', 10) == 0:
+            if 'output' not in params:
+                params['output'] = monitor('residual')
+            mon = params['output']
+            self.calcrhs(self.Qn)
+            value = self.modeldisc.all_L2average(self.residual)
+            mon.append(it=self._nit, time=self._time, value=value)
+
     def propagator(self, z):
         """computes scalar complex propagator of one time step
 
@@ -188,11 +293,17 @@ class timemodel:
         self.modeldisc = fakedisc(z)
         # make virtual field
         f = field.fdata(fakemodel(), fakemesh(), [0 * z + 1.0])
-        self.step(f, dt=1.0) # one step with normalized time step
+        self.step(f, dtloc=1.0) # one step with normalized time step
         # get back actual modeldisc
         self.modeldisc = saved_model
         return f.data[0]
 
+    def cflmax(self):
+        """estimation of maximum cfl, may not converge
+        """
+        def _gain_imag(sigma):
+            return abs(self.propagator(1j*sigma))-1.
+        return newton(_gain_imag, 10.)
 
 # --------------------------------------------------------------------
 
@@ -234,8 +345,8 @@ class rkmodel(timemodel):
     Returns:
 
     """
-    def __init__(self, mesh, modeldisc):
-        timemodel.__init__(self, mesh, modeldisc)
+    def __init__(self, mesh, modeldisc, monitors={}):
+        timemodel.__init__(self, mesh, modeldisc, monitors)
         self.check()
 
     def check(self):
@@ -331,7 +442,7 @@ class rk3_heun(rkmodel):
 # LOW STORAGE RUNGE KUTTA MODELS
 # --------------------------------------------------------------------
 
-class LSrkmodelHH(rkmodel):
+class LSrkmodelHH(timemodel):
     """generic implementation of LOW-STORAGE Runge-Kutta method
 
     Hu and Hussaini (JCP, 1996) method needs p-1 coefficients (_beta)
@@ -343,9 +454,9 @@ class LSrkmodelHH(rkmodel):
     Returns:
 
     """
-    # def __init__(self, mesh, modeldisc):
-    #     timemodel.__init__(self, mesh, modeldisc)
-    #     self.check()
+    def __init__(self, mesh, modeldisc, monitors={}):
+        timemodel.__init__(self, mesh, modeldisc, monitors)
+        self.check()
 
     def check(self):
         """check butcher array and define some algorithm properties"""
@@ -582,11 +693,10 @@ class gear(trapezoidal):
 #             ]
 #         )
 
-
 # --------------------------------------------------------------------
 # for tests
 
 List_LSRK_Integrators = [ lsrk25bb, lsrk26bb ]
 List_RK_Integrators = [ rk2, rk2_heun, rk3_heun, rk3ssp, rk4 ] + List_LSRK_Integrators
 List_Explicit_Integrators = [ explicit ] + List_RK_Integrators
-List_Implicit_Integrators = [ implicit, cranknicolson, gear ]
+List_Implicit_Integrators = [ implicit, cranknicolson, gear ]
\ No newline at end of file
