@@ -21,15 +21,23 @@
 class methoddict():
     """decorator to register decorated method as specific and tagged in the class model
     """
-    def __init__(self):
-        self.dict = {}
+    def __init__(self, items={}):
+        self.dict = dict(items)
 
     def register(self, name):
         def decorator(classmeth):
             self.dict[name] = classmeth
             return classmeth
         return decorator
-        
+    
+    def merge(self, mdict):
+        self.dict.update(mdict.dict)
+
+    def update(self, ddict: dict):
+        self.dict.update(ndict)
+    
+    def copy(self):
+        return methoddict(self.dict)
 
 # ===============================================================
 # implementation of MODEL class
@@ -57,7 +65,7 @@ class model():
         self.has_secondorder_terms = 0
         self.has_source_terms      = 0
         self._vardict = { }
-        #self._bcdict  = { 'dirichlet': self.bc_dirichlet }
+        self._bcdict  = model._bcdict.copy()
 
     def __repr__(self):
         print("model: ", self.equation)
